/*! \file exif-utils.h
 *  \brief EXIF data manipulation functions and types 
 */
/*
 * Copyright (c) 2001 Lutz Mueller <lutz@users.sourceforge.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details. 
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301  USA.
 */

#ifndef __EXIF_UTILS_H__
#define __EXIF_UTILS_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#if (defined(__MSHORT__) || defined(__PUREC__) || defined(__AHCC__)) && defined(EXIF_SLB) && (defined(__atarist__) || defined(__TOS__))
typedef long exif_int_t;
typedef unsigned long exif_uint_t;
#define exif_enum(x) exif_int_t
#else
typedef int exif_int_t;
typedef unsigned int exif_uint_t;
#define exif_enum(x) x
#endif
#if defined(EXIF_SLB) && (defined(__atarist__) || defined(__TOS__))
#  include <features.h>
#  define EXIF_CALLBACK __CDECL
#  define EXIF_API(x) x __CDECL
#else
#  define EXIF_API(x) x
#  define EXIF_CALLBACK
#endif

#include <libexif/exifbyte.h>
#include <libexif/exifform.h>
#include <stdint.h>

/*! EXIF Unsigned Byte data type */
typedef unsigned char	ExifByte;          /* 1 byte  */
	
/*! EXIF Signed Byte data type */
typedef signed char	ExifSByte;         /* 1 byte  */
	
/*! EXIF Text String data type */
typedef char *		ExifAscii;
	
/*! EXIF Unsigned Short data type */
typedef uint16_t	ExifShort;         /* 2 bytes */
	
/*! EXIF Signed Short data type */
typedef int16_t         ExifSShort;        /* 2 bytes */
	
/*! EXIF Unsigned Long data type */
typedef uint32_t	ExifLong;          /* 4 bytes */
	
/*! EXIF Signed Long data type */
typedef int32_t		ExifSLong;         /* 4 bytes */

/*! EXIF Unsigned Rational data type */
typedef struct {ExifLong numerator; ExifLong denominator;} ExifRational;

typedef char		ExifUndefined;     /* 1 byte  */


/*! EXIF Signed Rational data type */
typedef struct {ExifSLong numerator; ExifSLong denominator;} ExifSRational;


/*! Retrieve an #ExifShort value from memory.
 *
 * \param[in] b pointer to raw EXIF value in memory
 * \param[in] order byte order of raw value
 * \return value
 */
EXIF_API(ExifShort)     exif_get_short     (const unsigned char *b, exif_enum(ExifByteOrder) order);

/*! Retrieve an #ExifSShort value from memory.
 *
 * \param[in] b pointer to raw EXIF value in memory
 * \param[in] order byte order of raw value
 * \return value
 */
EXIF_API(ExifSShort)    exif_get_sshort    (const unsigned char *b, exif_enum(ExifByteOrder) order);

/*! Retrieve an #ExifLong value from memory.
 *
 * \param[in] b pointer to raw EXIF value in memory
 * \param[in] order byte order of raw value
 * \return value
 */
EXIF_API(ExifLong)      exif_get_long      (const unsigned char *b, exif_enum(ExifByteOrder) order);

/*! Retrieve an #ExifSLong value from memory.
 *
 * \param[in] b pointer to raw EXIF value in memory
 * \param[in] order byte order of raw value
 * \return value
 */
EXIF_API(ExifSLong)     exif_get_slong     (const unsigned char *b, exif_enum(ExifByteOrder) order);

/*! Retrieve an #ExifRational value from memory.
 *
 * \param[in] b pointer to raw EXIF value in memory
 * \param[in] order byte order of raw value
 * \return value
 */
EXIF_API(ExifRational)  exif_get_rational  (const unsigned char *b, exif_enum(ExifByteOrder) order);

/*! Retrieve an #ExifSRational value from memory.
 *
 * \param[in] b pointer to raw EXIF value in memory
 * \param[in] order byte order of raw value
 * \return value
 */
EXIF_API(ExifSRational) exif_get_srational (const unsigned char *b, exif_enum(ExifByteOrder) order);

/*! Store an ExifShort value into memory in EXIF format.
 *
 * \param[out] b buffer in which to write raw value
 * \param[in] order byte order to use
 * \param[in] value data value to store
 */
EXIF_API(void) exif_set_short(unsigned char *b, exif_enum(ExifByteOrder) order, exif_uint_t value);

/*! Store an ExifSShort value into memory in EXIF format.
 *
 * \param[out] b buffer in which to write raw value
 * \param[in] order byte order to use
 * \param[in] value data value to store
 */
EXIF_API(void) exif_set_sshort(unsigned char *b, exif_enum(ExifByteOrder) order, exif_int_t value);

/*! Store an ExifLong value into memory in EXIF format.
 *
 * \param[out] b buffer in which to write raw value
 * \param[in] order byte order to use
 * \param[in] value data value to store
 */
EXIF_API(void) exif_set_long(unsigned char *b, exif_enum(ExifByteOrder) order, ExifLong value);

/*! Store an ExifSLong value into memory in EXIF format.
 *
 * \param[out] b buffer in which to write raw value
 * \param[in] order byte order to use
 * \param[in] value data value to store
 */
EXIF_API(void) exif_set_slong(unsigned char *b, exif_enum(ExifByteOrder) order, ExifSLong value);

/*! Store an ExifRational value into memory in EXIF format.
 *
 * \param[out] b buffer in which to write raw value
 * \param[in] order byte order to use
 * \param[in] value data value to store
 */
EXIF_API(void) exif_set_rational(unsigned char *b, exif_enum(ExifByteOrder) order, const ExifRational *value);

/*! Store an ExifSRational value into memory in EXIF format.
 *
 * \param[out] b buffer in which to write raw value
 * \param[in] order byte order to use
 * \param[in] value data value to store
 */
EXIF_API(void) exif_set_srational(unsigned char *b, exif_enum(ExifByteOrder order), const ExifSRational *value);

/*! \internal */
EXIF_API(void) exif_convert_utf16_to_utf8 (char *out, const unsigned short *in, int32_t maxlen);

/* Please do not use this function outside of the library. */

/*! \internal */
EXIF_API(void) exif_array_set_byte_order (exif_enum(ExifFormat), unsigned char *, uint32_t, exif_enum(ExifByteOrder) o_orig, exif_enum(ExifByteOrder) o_new);

#undef  MIN
#define MIN(a, b)  (((a) < (b)) ? (a) : (b))

#undef  MAX
#define MAX(a, b)  (((a) > (b)) ? (a) : (b))

/* For compatibility with older versions */

/*! \deprecated Use EXIF_TAG_SUB_SEC_TIME instead. */
#define EXIF_TAG_SUBSEC_TIME EXIF_TAG_SUB_SEC_TIME

#ifdef __cplusplus
}
#endif /* __cplusplus */

#if defined(EXIF_SLB) && (defined(__atarist__) || defined(__TOS__))
#include <slb/exif.h>
#endif

#endif /* __EXIF_UTILS_H__ */
